@@ -1,23 +1,24 @@
 /*
- $Id: commands.c,v 1.8 2007-06-05 19:58:03+01 taviso Exp taviso $
+    Registration and general execution of commands.
 
- Copyright (C) 2006,2007,2009 Tavis Ormandy <taviso@sdf.lonestar.org>
- Copyright (C) 2009           Eli Dupree <elidupree@charter.net>
- Copyright (C) 2009,2010      WANG Lu <coolwanglu@gmail.com>
+    Copyright (C) 2006,2007,2009 Tavis Ormandy <taviso@sdf.lonestar.org>
+    Copyright (C) 2009           Eli Dupree <elidupree@charter.net>
+    Copyright (C) 2009,2010      WANG Lu <coolwanglu@gmail.com>
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
- (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    This file is part of libscanmem.
+
+    This library is free software: you can redistribute it and/or modify
+    it under the terms of the GNU Lesser General Public License as published
+    by the Free Software Foundation; either version 3 of the License, or
+    (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public License
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef _GNU_SOURCE
@@ -35,13 +36,11 @@
 #include <stdbool.h>
 #include <ctype.h>
 
-#include <readline/readline.h>
-
 #include "commands.h"
 #include "show_message.h"
 
 /*
- * registercommand - add the command and a pointer to its handler to the commands list.
+ * sm_registercommand - add the command and a pointer to its handler to the commands list.
  *
  * so that free(data) works when destroying the list, I just concatenate the string
  * with the command structure. I could have used a static vector of commands, but this
@@ -49,9 +48,8 @@
  *
  */
 
-/*lint -esym(818,handler) dont declare handler const */
-bool registercommand(const char *command, void *handler, list_t * commands,
-                     char *shortdoc, char *longdoc)
+bool sm_registercommand(const char *command, void *handler, list_t *commands,
+                        char *shortdoc, char *longdoc)
 {
     command_t *data;
 
@@ -87,7 +85,7 @@ bool registercommand(const char *command, void *handler, list_t * commands,
     return true;
 }
 
-bool execcommand(globals_t * vars, const char *commandline)
+bool sm_execcommand(globals_t *vars, const char *commandline)
 {
     unsigned argc;
     char *str = NULL, *tok = NULL;
