@@ -1,23 +1,25 @@
 /*
- $Id: handlers.c,v 1.12 2007-06-05 01:45:34+01 taviso Exp taviso $
+    Specific command handling.
 
- Copyright (C) 2006,2007,2009 Tavis Ormandy <taviso@sdf.lonestar.org>
- Copyright (C) 2009           Eli Dupree <elidupree@charter.net>
- Copyright (C) 2009,2010      WANG Lu <coolwanglu@gmail.com>
+    Copyright (C) 2006,2007,2009 Tavis Ormandy <taviso@sdf.lonestar.org>
+    Copyright (C) 2009           Eli Dupree <elidupree@charter.net>
+    Copyright (C) 2009,2010      WANG Lu <coolwanglu@gmail.com>
+    Copyright (C) 2014-2016      Sebastian Parschauer <s.parschauer@gmx.de>
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
- (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    This file is part of libscanmem.
+
+    This library is free software: you can redistribute it and/or modify
+    it under the terms of the GNU Lesser General Public License as published
+    by the Free Software Foundation; either version 3 of the License, or
+    (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public License
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef _GNU_SOURCE
@@ -35,7 +37,7 @@
 #include <assert.h>
 #include <setjmp.h>
 #include <alloca.h>
-#include <strings.h>            /*lint -esym(526,strcasecmp) */
+#include <strings.h>
 #include <string.h>
 #include <stdbool.h>
 #include <time.h>
@@ -43,18 +45,16 @@
 #include <limits.h>            /* to determine the word width */
 #include <errno.h>
 #include <inttypes.h>
-
-#include <readline/readline.h>
+#include <ctype.h>
 
 #include "commands.h"
+#include "endianness.h"
 #include "handlers.h"
 #include "interrupt.h"
 #include "show_message.h"
 
 #define USEPARAMS() ((void) vars, (void) argv, (void) argc)     /* macro to hide gcc unused warnings */
 
-/*lint -esym(818, vars, argv) dont want want to declare these as const */
-
 /*
  * This file defines all the command handlers used, each one is registered using
  * registercommand(), and when a matching command is entered, the commandline is
@@ -72,6 +72,18 @@
 
 #define calloca(x,y) (memset(alloca((x) * (y)), 0x00, (x) * (y)))
 
+/* try to determine the size of a pointer */
+#ifndef ULONG_MAX
+#warning ULONG_MAX is not defined!
+#endif
+#if ULONG_MAX == 4294967295UL
+#define POINTER_FMT "%8lx"
+#elif ULONG_MAX == 18446744073709551615UL
+#define POINTER_FMT "%12lx"
+#else
+#define POINTER_FMT "%12lx"
+#endif
+
 bool handler__set(globals_t * vars, char **argv, unsigned argc)
 {
     unsigned block, seconds = 1;
@@ -170,8 +182,9 @@ bool handler__set(globals_t * vars, char **argv, unsigned argc)
     if (INTERRUPTABLE()) {
         
         /* control returns here when interrupted */
-        free(settings);
-        detach(vars->target);
+// settings is allocated with alloca, do not free it
+//        free(settings);
+        sm_detach(vars->target);
         ENDINTERRUPTABLE();
         return true;
     }
@@ -235,13 +248,15 @@ bool handler__set(globals_t * vars, char **argv, unsigned argc)
                         /* copy val onto v */
                         /* XXX: valcmp? make sure the sizes match */
                         old = data_to_val(loc.swath, loc.index /* ,MATCHES_AND_VALUES */);
+                        zero_value(&v);
                         v.flags = old.flags = loc.swath->data[loc.index].match_info;
                         uservalue2value(&v, &userval);
                         
-                        show_info("setting *%p to %#"PRIx64"...\n", address, v.int64_value); 
+                        show_info("setting *%p to %#"PRIx64"...\n", address, v.int64_value);
 
                         /* set the value specified */
-                        if (setaddr(vars->target, address, &v) == false) {
+                        fix_endianness(vars, &v);
+                        if (sm_setaddr(vars->target, address, &v) == false) {
                             show_error("failed to set a value.\n");
                             goto fail;
                         }
@@ -269,12 +284,14 @@ bool handler__set(globals_t * vars, char **argv, unsigned argc)
                                     
                         value_t old = data_to_val(reading_swath_index, reading_iterator /* ,MATCHES_AND_VALUES */);
                         value_t v;
+                        zero_value(&v);
                         v.flags = old.flags = reading_swath_index->data[reading_iterator].match_info;
                         uservalue2value(&v, &userval);
 
-                        show_info("setting *%p to %"PRIx64"...\n", address, v.int64_value); 
+                        show_info("setting *%p to %#"PRIx64"...\n", address, v.int64_value);
 
-                        if (setaddr(vars->target, address, &v) == false) {
+                        fix_endianness(vars, &v);
+                        if (sm_setaddr(vars->target, address, &v) == false) {
                             show_error("failed to set a value.\n");
                             goto fail;
                         }
@@ -313,10 +330,11 @@ fail:
 /* FORMAT (don't change, front-end depends on this): 
  * [#no] addr, value, [possible types (separated by space)]
  */
-bool handler__list(globals_t * vars, char **argv, unsigned argc)
+bool handler__list(globals_t *vars, char **argv, unsigned argc)
 {
     unsigned i = 0;
     int buf_len = 128; /* will be realloc later if necessary */
+    element_t *np = NULL;
     char *v = malloc(buf_len);
     if (v == NULL)
     {
@@ -328,8 +346,11 @@ bool handler__list(globals_t * vars, char **argv, unsigned argc)
 
     USEPARAMS();
 
-    if(!(vars->matches))
-        return true;
+    if (!vars->matches)
+        goto out_free;
+
+    if (vars->regions)
+        np = vars->regions->head;
 
     matches_and_old_values_swath *reading_swath_index = (matches_and_old_values_swath *)vars->matches->swaths;
     int reading_iterator = 0;
@@ -342,7 +363,7 @@ bool handler__list(globals_t * vars, char **argv, unsigned argc)
         /* Only actual matches are considered */
         if (flags_to_max_width_in_bytes(flags) > 0)
         {
-            switch(globals.options.scan_data_type)
+            switch(vars->options.scan_data_type)
             {
             case BYTEARRAY:
                 ; /* cheat gcc */ 
@@ -376,34 +397,46 @@ bool handler__list(globals_t * vars, char **argv, unsigned argc)
                 value_t val = data_to_val(reading_swath_index, reading_iterator /* ,MATCHES_AND_VALUES */);
                 truncval_to_flags(&val, flags);
 
-                if (valtostr(&val, v, buf_len) != true) {
-                    strncpy(v, "unknown", buf_len);
-                }
+                valtostr(&val, v, buf_len);
                 break;
             }
 
-/* try to determine the size of a pointer */
-#if ULONGMAX == 4294967295UL
-#define POINTER_FMT "%10p"
-#elif ULONGMAX == 18446744073709551615UL
-#define POINTER_FMT "%20p"
-#else
-#define POINTER_FMT "%20p"
-#endif
-
-            fprintf(stdout, "[%2u] "POINTER_FMT", %s\n", i++, remote_address_of_nth_element(reading_swath_index, reading_iterator /* ,MATCHES_AND_VALUES */), v);
+            void *address = remote_address_of_nth_element(reading_swath_index,
+                reading_iterator /* ,MATCHES_AND_VALUES */);
+            unsigned long address_ul = (unsigned long)address;
+            int region_id = 99;
+            unsigned long match_off = 0;
+            const char *region_type = "??";
+            /* get region info belonging to the match -
+             * note: we assume the regions list and matches to be sorted
+             */
+            while (np) {
+                region_t *region = np->data;
+                unsigned long region_start = (unsigned long)region->start;
+                if (address_ul < region_start + region->size &&
+                  address_ul >= region_start) {
+                    region_id = region->id;
+                    match_off = address_ul - region->load_addr;
+                    region_type = region_type_names[region->type];
+                    break;
+                }
+                np = np->next;
+            }
+            fprintf(stdout, "[%2u] "POINTER_FMT", %2u + "POINTER_FMT", %5s, %s\n", 
+                    i++, address_ul, region_id, match_off, region_type, v);
         }
-	
+
         /* Go on to the next one... */
         ++reading_iterator;
         if (reading_iterator >= reading_swath_index->number_of_bytes)
         {
-            assert(((matches_and_old_values_swath *)(local_address_beyond_last_element(reading_swath_index /* ,MATCHES_AND_VALUES */)))->number_of_bytes >= 0);
-            reading_swath_index = local_address_beyond_last_element(reading_swath_index /* ,MATCHES_AND_VALUES */);
+            reading_swath_index = local_address_beyond_last_element(
+                reading_swath_index /* ,MATCHES_AND_VALUES */);
             reading_iterator = 0;
         }
     }
 
+out_free:
     free(v);
     return true;
 }
@@ -433,8 +466,10 @@ bool handler__delete(globals_t * vars, char **argv, unsigned argc)
     
     if (loc.swath)
     {
-        /* It is not convenient to check whether anything else relies on this, so just mark it as not a REAL match */
-        memset(&(loc.swath->data[loc.index].match_info), 0, sizeof(match_flags));
+        /* It is not convenient to check whether anything else relies on this,
+           so just mark it as not a REAL match */
+        zero_match_flags(&loc.swath->data[loc.index].match_info);
+        vars->num_matches--;
         return true;
     }
     else
@@ -448,7 +483,6 @@ bool handler__delete(globals_t * vars, char **argv, unsigned argc)
 
 bool handler__reset(globals_t * vars, char **argv, unsigned argc)
 {
-
     USEPARAMS();
 
     if (vars->matches) { free(vars->matches); vars->matches = NULL; vars->num_matches = 0; }
@@ -463,7 +497,7 @@ bool handler__reset(globals_t * vars, char **argv, unsigned argc)
     }
 
     /* read in maps if a pid is known */
-    if (vars->target && readmaps(vars->target, vars->regions) != true) {
+    if (vars->target && sm_readmaps(vars->target, vars->regions) != true) {
         show_error("sorry, there was a problem getting a list of regions to search.\n");
         show_warn("the pid may be invalid, or you don't have permission.\n");
         vars->target = 0;
@@ -497,7 +531,7 @@ bool handler__pid(globals_t * vars, char **argv, unsigned argc)
     return handler__reset(vars, resetargv, 1);
 }
 
-bool handler__snapshot(globals_t * vars, char **argv, unsigned argc)
+bool handler__snapshot(globals_t *vars, char **argv, unsigned argc)
 {
     USEPARAMS();
     
@@ -511,7 +545,7 @@ bool handler__snapshot(globals_t * vars, char **argv, unsigned argc)
     /* remove any existing matches */
     if (vars->matches) { free(vars->matches); vars->matches = NULL; vars->num_matches = 0; }
 
-    if (searchregions(vars, MATCHANY, NULL) != true) {
+    if (sm_searchregions(vars, MATCHANY, NULL) != true) {
         show_error("failed to save target address space.\n");
         return false;
     }
@@ -520,7 +554,7 @@ bool handler__snapshot(globals_t * vars, char **argv, unsigned argc)
 }
 
 /* dregion [!][x][,x,...] */
-bool handler__dregion(globals_t * vars, char **argv, unsigned argc)
+bool handler__dregion(globals_t *vars, char **argv, unsigned argc)
 {
     unsigned id;
     bool invert = false;
@@ -632,6 +666,7 @@ bool handler__dregion(globals_t * vars, char **argv, unsigned argc)
             if (l_append(keep, keep->tail, save) == -1) {
                 show_error("sorry, there was an internal memory error.\n");
                 free(save);
+                l_destroy(keep);
                 return false;
             }
             continue;
@@ -700,8 +735,10 @@ bool handler__lregions(globals_t * vars, char **argv, unsigned argc)
     while (np) {
         region_t *region = np->data;
 
-        fprintf(stderr, "[%2u] %#10lx, %7lu bytes, %c%c%c, %s\n",
-                region->id, (unsigned long)region->start, region->size,
+        fprintf(stderr, "[%2u] "POINTER_FMT", %7lu bytes, %5s, "POINTER_FMT", %c%c%c, %s\n", 
+                region->id,
+                (unsigned long)region->start, region->size,
+                region_type_names[region->type], region->load_addr,
                 region->flags.read ? 'r' : '-',
                 region->flags.write ? 'w' : '-',
                 region->flags.exec ? 'x' : '-',
@@ -720,9 +757,9 @@ bool handler__decinc(globals_t * vars, char **argv, unsigned argc)
 
     USEPARAMS();
 
-    if(argc == 1)
+    if (argc == 1)
     {
-        memset(&val, 0x00, sizeof(val));
+        zero_uservalue(&val);
     }
     else if (argc > 2)
     {
@@ -740,7 +777,7 @@ bool handler__decinc(globals_t * vars, char **argv, unsigned argc)
 
     if (strcmp(argv[0], "=") == 0)
     {
-        m = MATCHNOTCHANGED;
+        m = (argc == 1) ? MATCHNOTCHANGED : MATCHEQUALTO;
     }
     else if (strcmp(argv[0], "!=") == 0)
     {
@@ -764,25 +801,34 @@ bool handler__decinc(globals_t * vars, char **argv, unsigned argc)
     }
     else
     {
-        show_error("unrecogised match type seen at decinc handler.\n");
+        show_error("unrecognized match type seen at decinc handler.\n");
         return false;
     }
 
     if (vars->matches) {
-        if (checkmatches(vars, m, &val) == false) {
+        if (sm_checkmatches(vars, m, &val) == false) {
             show_error("failed to search target address space.\n");
             return false;
         }
     } else {
-        /* < > = != cannot be the initial scan */
-        if (argc == 1)
+        /* Cannot be used on first scan:
+         *   =, !=, <, >, +, + N, -, - N
+         * Can be used on first scan:
+         *   = N, != N, < N, > N
+         */
+        if (m == MATCHNOTCHANGED  ||
+            m == MATCHCHANGED     ||
+            m == MATCHDECREASED   ||
+            m == MATCHINCREASED   ||
+            m == MATCHDECREASEDBY ||
+            m == MATCHINCREASEDBY )
         {
             show_error("cannot use that search without matches\n");
             return false;
         }
         else
         {
-            if (searchregions(vars, m, &val) != true) {
+            if (sm_searchregions(vars, m, &val) != true) {
                 show_error("failed to search target address space.\n");
                 return false;
             }
@@ -797,11 +843,11 @@ bool handler__decinc(globals_t * vars, char **argv, unsigned argc)
     return true;
 }
 
-bool handler__version(globals_t * vars, char **argv, unsigned argc)
+bool handler__version(globals_t *vars, char **argv, unsigned argc)
 {
     USEPARAMS();
 
-    printversion(stderr);
+    vars->printversion(stderr);
     return true;
 }
 
@@ -836,13 +882,13 @@ bool handler__string(globals_t * vars, char **argv, unsigned argc)
     /* user has specified an exact value of the variable to find */
     if (vars->matches) {
         /* already know some matches */
-        if (checkmatches(vars, MATCHEQUALTO, &val) != true) {
+        if (sm_checkmatches(vars, MATCHEQUALTO, &val) != true) {
             show_error("failed to search target address space.\n");
             return false;
         }
     } else {
         /* initial search */
-        if (searchregions(vars, MATCHEQUALTO, &val) != true) {
+        if (sm_searchregions(vars, MATCHEQUALTO, &val) != true) {
             show_error("failed to search target address space.\n");
             return false;
         }
@@ -857,11 +903,26 @@ bool handler__string(globals_t * vars, char **argv, unsigned argc)
     return true;
 }
 
+static inline bool parse_uservalue_default(char *str, uservalue_t *val)
+{
+    bool ret = true;
+
+    if (!parse_uservalue_number(str, val)) {
+        show_error("unable to parse number `%s`\n", str);
+        ret = false;
+    }
+    return ret;
+}
+
 bool handler__default(globals_t * vars, char **argv, unsigned argc)
 {
-    uservalue_t val;
-    bool ret;
+    uservalue_t vals[2];
+    uservalue_t *val = &vals[0];
     bytearray_element_t *array = NULL;
+    scan_match_type_t m = MATCHEQUALTO;
+    char *ustr = argv[0];
+    char *pos;
+    bool ret = false;
 
     USEPARAMS();
 
@@ -880,13 +941,21 @@ bool handler__default(globals_t * vars, char **argv, unsigned argc)
         if (argc != 1)
         {
             show_error("unknown command\n");
-            ret = false;
             goto retl;
         }
-        if (!parse_uservalue_number(argv[0], &val)) {
-            show_error("unable to parse command `%s`\n", argv[0]);
-            ret = false;
-            goto retl;
+	/* detect a range */
+	pos = strstr(ustr, "..");
+	if (pos) {
+            *pos = '\0';
+            if (!parse_uservalue_default(ustr, &vals[0]))
+                goto retl;
+            ustr = pos + 2;
+            if (!parse_uservalue_default(ustr, &vals[1]))
+                goto retl;
+            m = MATCHRANGE;
+        } else {
+            if (!parse_uservalue_default(ustr, val))
+                goto retl;
         }
         break;
     case BYTEARRAY:
@@ -896,18 +965,16 @@ bool handler__default(globals_t * vars, char **argv, unsigned argc)
         if (array == NULL)
         {
             show_error("there's a memory allocation error.\n");
-            ret = false;
             goto retl;
         }
-        if (!parse_uservalue_bytearray(argv, argc, array, &val)) {
-            show_error("unable to parse command `%s`\n", argv[0]);
-            ret = false;
+        if (!parse_uservalue_bytearray(argv, argc, array, val)) {
+            show_error("unable to parse command `%s`\n", ustr);
             goto retl;
         }
         break;
     case STRING:
-        show_error("unable to parse command `%s`\nIf you want to scan for a string, use command `\"`.\n", argv[0]);
-        ret = false;
+        show_error("unable to parse command `%s`\nIf you want to scan"
+                   " for a string, use command `\"`.\n", ustr);
         goto retl;
         break;
     default:
@@ -917,23 +984,20 @@ bool handler__default(globals_t * vars, char **argv, unsigned argc)
 
     /* need a pid for the rest of this to work */
     if (vars->target == 0) {
-        ret = false;
         goto retl;
     }
 
     /* user has specified an exact value of the variable to find */
     if (vars->matches) {
         /* already know some matches */
-        if (checkmatches(vars, MATCHEQUALTO, &val) != true) {
+        if (sm_checkmatches(vars, m, val) != true) {
             show_error("failed to search target address space.\n");
-            ret = false;
             goto retl;
         }
     } else {
         /* initial search */
-        if (searchregions(vars, MATCHEQUALTO, &val) != true) {
+        if (sm_searchregions(vars, m, val) != true) {
             show_error("failed to search target address space.\n");
-            ret = false;
             goto retl;
         }
     }
@@ -953,12 +1017,12 @@ retl:
     return ret;
 }
 
-bool handler__update(globals_t * vars, char **argv, unsigned argc)
+bool handler__update(globals_t *vars, char **argv, unsigned argc)
 {
 
     USEPARAMS();
     if (vars->matches) {
-        if (checkmatches(vars, MATCHANY, NULL) == false) {
+        if (sm_checkmatches(vars, MATCHANY, NULL) == false) {
             show_error("failed to scan target address space.\n");
             return false;
         }
@@ -970,7 +1034,7 @@ bool handler__update(globals_t * vars, char **argv, unsigned argc)
     return true;
 }
 
-bool handler__exit(globals_t * vars, char **argv, unsigned argc)
+bool handler__exit(globals_t *vars, char **argv, unsigned argc)
 {
     USEPARAMS();
 
@@ -980,7 +1044,7 @@ bool handler__exit(globals_t * vars, char **argv, unsigned argc)
 
 #define DOC_COLUMN  11           /* which column descriptions start on with help command */
 
-bool handler__help(globals_t * vars, char **argv, unsigned argc)
+bool handler__help(globals_t *vars, char **argv, unsigned argc)
 {
     bool ret = false;
     list_t *commands = vars->commands;
@@ -1000,8 +1064,10 @@ bool handler__help(globals_t * vars, char **argv, unsigned argc)
     }
 
     /* print version information for generic help */
-    if (argv[1] == NULL)
-        printversion(outfd);
+    if (argv[1] == NULL) {
+        vars->printversion(outfd);
+        fprintf(outfd, "\n");
+    }
 
     /* traverse the commands list, printing out the relevant documentation */
     while (np) {
@@ -1085,12 +1151,14 @@ bool handler__shell(globals_t * vars, char **argv, unsigned argc)
 
     /* finally execute command */
     if (system(command) == -1) {
-        free(command);
+// command is allocated with alloca, do not free it
+//        free(command);
         show_error("system() failed, command was not executed.\n");
         return false;
     }
 
-    free(command);
+// command is allocated with alloca, do not free it
+//    free(command);
     return true;
 }
 
@@ -1103,11 +1171,16 @@ bool handler__watch(globals_t * vars, char **argv, unsigned argc)
     match_location loc;
     value_t old_val;
     void *address;
+    scan_data_type_t data_type = vars->options.scan_data_type;
 
     if (argc != 2) {
         show_error("was expecting one argument, see `help watch`.\n");
         return false;
     }
+    if ((data_type == BYTEARRAY) || (data_type == STRING)) {
+        show_error("`watch` is not supported for bytearray or string.\n");
+        return false;
+    }
 
     /* parse argument */
     id = strtoul(argv[1], &end, 0x00);
@@ -1131,15 +1204,14 @@ bool handler__watch(globals_t * vars, char **argv, unsigned argc)
     address = remote_address_of_nth_element(loc.swath, loc.index /* ,MATCHES_AND_VALUES */);
     
     old_val = data_to_val(loc.swath, loc.index /* ,MATCHES_AND_VALUES */);
+    old_val.flags = loc.swath->data[loc.index].match_info;
     valcpy(&o, &old_val);
     valcpy(&n, &o);
 
-    if (valtostr(&o, buf, sizeof(buf)) == false) {
-        strncpy(buf, "unknown", sizeof(buf));
-    }
+    valtostr(&o, buf, sizeof(buf));
 
     if (INTERRUPTABLE()) {
-        (void) detach(vars->target);
+        (void) sm_detach(vars->target);
         ENDINTERRUPTABLE();
         return true;
     }
@@ -1152,10 +1224,10 @@ bool handler__watch(globals_t * vars, char **argv, unsigned argc)
 
     while (true) {
 
-        if (attach(vars->target) == false)
+        if (sm_attach(vars->target) == false)
             return false;
 
-        if (peekdata(vars->target, address, &n) == false)
+        if (sm_peekdata(vars->target, address, &n) == false)
             return false;
 
 
@@ -1163,16 +1235,14 @@ bool handler__watch(globals_t * vars, char **argv, unsigned argc)
 
         /* check if the new value is different */
         match_flags tmpflags;
-        memset(&tmpflags, 0x00, sizeof(tmpflags));
-        scan_routine_t valuecmp_routine = (get_scanroutine(ANYNUMBER, MATCHCHANGED));
+        zero_match_flags(&tmpflags);
+        scan_routine_t valuecmp_routine = (sm_get_scanroutine(ANYNUMBER, MATCHCHANGED));
         if (valuecmp_routine(&o, &n, NULL, &tmpflags, address)) {
 
             valcpy(&o, &n);
             truncval(&o, &old_val);
 
-            if (valtostr(&o, buf, sizeof(buf)) == false) {
-                strncpy(buf, "unknown", sizeof(buf));
-            }
+            valtostr(&o, buf, sizeof(buf));
 
             /* fetch new timestamp */
             t = time(NULL);
@@ -1183,7 +1253,7 @@ bool handler__watch(globals_t * vars, char **argv, unsigned argc)
         }
 
         /* detach after valuecmp_routine, since it may read more data (e.g. bytearray) */
-        detach(vars->target);
+        sm_detach(vars->target);
 
         (void) sleep(1);
     }
@@ -1205,7 +1275,7 @@ bool handler__show(globals_t * vars, char **argv, unsigned argc)
     else if (strcmp(argv[1], "warranty") == 0)
         show_user(SM_WARRANTY);
     else if (strcmp(argv[1], "version") == 0)
-        printversion(stderr);
+        vars->printversion(stderr);
     else {
         show_error("unrecognized show command `%s`\n", argv[1]);
         return false;
@@ -1261,12 +1331,16 @@ bool handler__dump(globals_t * vars, char **argv, unsigned argc)
     buf = malloc(len + sizeof(long));
     if (buf == NULL)
     {
+        if (dump_f)
+            fclose(dump_f);
         show_error("memory allocation failed.\n");
         return false;
     }
 
-    if (!read_array(vars->target, addr, buf, len))
+    if (!sm_read_array(vars->target, addr, buf, len))
     {
+        if (dump_f)
+            fclose(dump_f);
         show_error("read memory failed.\n");
         free(buf);
         return false;
@@ -1463,6 +1537,9 @@ bool handler__write(globals_t * vars, char **argv, unsigned argc)
             ret = false;
             goto retl;
         }
+        if (1 < data_width && vars->options.reverse_endianness) {
+            swap_bytes_var(buf, data_width);
+        }
         break;
     case 1: // bytearray
         ; /* cheat gcc */
@@ -1497,7 +1574,7 @@ bool handler__write(globals_t * vars, char **argv, unsigned argc)
             }
             if (wildcard_used)
             {
-                if(!read_array(vars->target, addr, buf, data_width))
+                if(!sm_read_array(vars->target, addr, buf, data_width))
                 {
                     show_error("read memory failed.\n");
                     free(array);
@@ -1525,7 +1602,7 @@ bool handler__write(globals_t * vars, char **argv, unsigned argc)
     }
 
     /* write into memory */
-    ret = write_array(vars->target, addr, buf, data_width);
+    ret = sm_write_array(vars->target, addr, buf, data_width);
 
 retl:
     if(buf)
@@ -1592,6 +1669,20 @@ bool handler__option(globals_t * vars, char **argv, unsigned argc)
             return false;
         }
     }
+    else if (strcasecmp(argv[1], "endianness") == 0)
+    {
+        // data is host endian: don't swap
+        if (strcmp(argv[2], "0") == 0) {vars->options.reverse_endianness = 0; }
+        // data is little endian: swap if host is big endian
+        else if (strcmp(argv[2], "1") == 0) {vars->options.reverse_endianness = big_endian; }
+        // data is big endian: swap if host is little endian
+        else if (strcmp(argv[2], "2") == 0) {vars->options.reverse_endianness = !big_endian; }
+        else
+        {
+            show_error("bad value for endianness, see `help option`.\n");
+            return false;
+        }
+    }
     else
     {
         show_error("unknown option specified, see `help option`.\n");
