@@ -1,9 +1,29 @@
 /*
-  $Id: handlers.h,v 1.13 2007-06-05 01:45:35+01 taviso Exp taviso $
+    Specific command handling with help texts.
+
+    Copyright (C) 2006,2007,2009 Tavis Ormandy <taviso@sdf.lonestar.org>
+    Copyright (C) 2009           Eli Dupree <elidupree@charter.net>
+    Copyright (C) 2009,2010      WANG Lu <coolwanglu@gmail.com>
+    Copyright (C) 2014-2016      Sebastian Parschauer <s.parschauer@gmx.de>
+
+    This file is part of libscanmem.
+
+    This library is free software: you can redistribute it and/or modify
+    it under the terms of the GNU Lesser General Public License as published
+    by the Free Software Foundation; either version 3 of the License, or
+    (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public License
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#ifndef _INC_HANDLERS
-#define _INC_HANDLERS           /* include guard */
+#ifndef HANDLERS_H
+#define HANDLERS_H
 
 /*
  * SHRTDOC's are one line descriptions of the supported command (shown with `help`).
@@ -39,7 +59,7 @@
                "\tset 0,3=42/8 - set matches 0 and 3 to 42 every 8 seconds\n" \
                "\tset 12,13,14=0x23/2 23,19=0/8 6=4 0 - complex example, can be combined" \
 
-bool handler__set(globals_t * vars, char **argv, unsigned argc);
+bool handler__set(globals_t *vars, char **argv, unsigned argc);
 
 #define LIST_SHRTDOC "list all currently known matches"
 #define LIST_LONGDOC "usage: list\n" \
@@ -47,9 +67,12 @@ bool handler__set(globals_t * vars, char **argv, unsigned argc);
                "match, such as its type, location, and last known value. The number in\n" \
                "the left column is the `match-id`, this can be passed to other commands\n" \
                "such as `set`, `delete`, etc.\n" \
-               "The flags displayed indicate the possible types of the variable\n" 
+               "The flags displayed indicate the possible types of the variable.\n" \
+               "Also the region id, an offset and the region type belonging to a match\n" \
+               "are displayed. The offset is used from the code load address or region start.\n" \
+               "This helps bypassing address space layout randomization (ASLR).\n"
 
-bool handler__list(globals_t * vars, char **argv, unsigned argc);
+bool handler__list(globals_t *vars, char **argv, unsigned argc);
 
 #define DELETE_SHRTDOC "delete a known match by match-id"
 #define DELETE_LONGDOC "usage: delete match-id\n" \
@@ -62,7 +85,7 @@ bool handler__list(globals_t * vars, char **argv, unsigned argc);
                 "\tdelete 0 - delete match 0\n" \
                 "NOTE: match-ids may be recalculated after matches are removed or added."
 
-bool handler__delete(globals_t * vars, char **argv, unsigned argc);
+bool handler__delete(globals_t *vars, char **argv, unsigned argc);
 
 #define RESET_SHRTDOC "forget all matches, and reinitialise regions"
 #define RESET_LONGDOC "usage: reset\n" \
@@ -70,7 +93,7 @@ bool handler__delete(globals_t * vars, char **argv, unsigned argc);
                 "maps file. Useful if you have made an error, or want to find a new\n" \
                 "variable.\n"
 
-bool handler__reset(globals_t * vars, char **argv, unsigned argc);
+bool handler__reset(globals_t *vars, char **argv, unsigned argc);
 
 #define PID_SHRTDOC "print current pid, or attach to a new process"
 #define PID_LONGDOC "usage: pid [pid]\n" \
@@ -78,17 +101,17 @@ bool handler__reset(globals_t * vars, char **argv, unsigned argc);
                 "process `pid`. If `pid` is not specified, print information about\n" \
                 "current process."
 
-bool handler__pid(globals_t * vars, char **argv, unsigned argc);
+bool handler__pid(globals_t *vars, char **argv, unsigned argc);
 
 #define SNAPSHOT_SHRTDOC "take a snapshot of the current process state"
 #define SNAPSHOT_LONGDOC "usage: snapshot\n" \
-                "Take a snapshot of the entire process in its current state, this is useful\n" \
-                "If you don't know the exact value of the variable you are searching for, but\n" \
+                "Take a snapshot of the entire process in its current state. This is useful\n" \
+                "if you don't know the exact value of the variable you are searching for, but\n" \
                 "can describe it in terms of higher, lower or equal (see commands `>`,`<` and\n" \
                 "`=`).\n\n" \
-                "NOTE: this can use a lot of memory with large processes."
+                "NOTE: This can use a lot of memory with large processes."
 
-bool handler__snapshot(globals_t * vars, char **argv, unsigned argc);
+bool handler__snapshot(globals_t *vars, char **argv, unsigned argc);
 
 #define DREGION_SHRTDOC "delete a known region by region-id"
 #define DREGION_LONGDOC "usage: dregion [!]region-id[,region-id[,...]]\n" \
@@ -96,37 +119,42 @@ bool handler__snapshot(globals_t * vars, char **argv, unsigned argc);
                 "affected from the match list. The `region-id` can be found using the `lregions`\n" \
                 "command. A leading `!` indicates the list should be inverted.\n" 
 
-bool handler__dregion(globals_t * vars, char **argv, unsigned argc);
+bool handler__dregion(globals_t *vars, char **argv, unsigned argc);
 
 #define LREGIONS_SHRTDOC "list all known regions"
 #define LREGIONS_LONGDOC "usage: lregions\n" \
                 "Print all the currently known regions, along with details such as the\n" \
-                "start address, size, permissions and associated filename. The number in\n" \
-                "the left column is the `region-id`, this can be passed to other commands\n" \
-                "that process regions, such as `dregion`."
+                "start address, size, region type, load address, permissions and associated\n" \
+                "filename. The number in the left column is the `region-id`, this can be\n" \
+                "passed to other commands that process regions, such as `dregion`.\n" \
+                "The load address is the start of the .text region for the executable\n" \
+                "or libraries. Otherwise, it is the region start.\n"
 
-bool handler__lregions(globals_t * vars, char **argv, unsigned argc);
+bool handler__lregions(globals_t *vars, char **argv, unsigned argc);
 
 #define GREATERTHAN_SHRTDOC "match values that have increased or greater than some number"
-#define LESSTHAN_SHRTDOC "match values that have decreased or less than some number"
-#define NOTCHANGED_SHRTDOC "match all variables that have changed since last scan"
-#define CHANGED_SHRTDOC "match all variables that have not changed since last scan"
-#define INCREASED_SHRTDOC "match values that have increased by some given number"
-#define DECREASED_SHRTDOC "match values that have decreased by some given number"
+#define LESSTHAN_SHRTDOC    "match values that have decreased or less than some number"
+#define NOTCHANGED_SHRTDOC  "match values that have not changed or equal to some number"
+#define CHANGED_SHRTDOC     "match values that have changed or different from some number"
+#define INCREASED_SHRTDOC   "match values that have increased at all or by some number"
+#define DECREASED_SHRTDOC   "match values that have decreased at all or by some number"
 
 #define GREATERTHAN_LONGDOC "usage: > [n]\n" \
                 "If n is given, match values that are greater than n.\n" \
-                "Otherwise all current matches that have not increased since the last scan are discarded. (same as `+`)\n" \
+                "Otherwise match all values that have increased. (same as `+`)\n" \
                 "You can use this in conjunction with `snapshot` if you never know its value."
 #define LESSTHAN_LONGDOC "usage: < [n]\n" \
                 "If n is given, match values that are less than n.\n" \
-                "Otherwise all current matches that have not decreased since the last scan are discarded.(same as `-`)\n" \
+                "Otherwise match all values that have decreased. (same as `-`)\n" \
                 "You can use this in conjunction with `snapshot` if you never know its value."
-#define NOTCHANGED_LONGDOC "usage: =\n" \
-                "All current matches that have changed since the last scan are discarded.\n" \
+
+#define NOTCHANGED_LONGDOC "usage: = [n]\n" \
+                "If n is given, match values that are equal to n. (same as `n`)\n" \
+                "Otherwise match all values that have not changed since the last scan.\n" \
                 "You can use this in conjunction with `snapshot` if you never know its value."
-#define CHANGED_LONGDOC "usage: !=\n" \
-                "All current matches that have not changed since the last scan are discarded.\n" \
+#define CHANGED_LONGDOC "usage: != [n]\n" \
+                "If n is given, match values that are different from n.\n" \
+                "Otherwise match all values that have changed since the last scan.\n" \
                 "You can use this in conjunction with `snapshot` if you never know its value."
 
 #define INCREASED_LONGDOC "usage: + [n]\n" \
@@ -140,19 +168,19 @@ bool handler__lregions(globals_t * vars, char **argv, unsigned argc);
                 "You can use this in conjunction with `snapshot` if you never know its value."
 
 
-bool handler__decinc(globals_t * vars, char **argv, unsigned argc);
+bool handler__decinc(globals_t *vars, char **argv, unsigned argc);
 
 #define VERSION_SHRTDOC "print current version"
 #define VERSION_LONGDOC "usage: version\n" \
                 "Display the current version of scanmem in use."
 
-bool handler__version(globals_t * vars, char **argv, unsigned argc);
+bool handler__version(globals_t *vars, char **argv, unsigned argc);
 
 #define EXIT_SHRTDOC "exit the program immediately"
 #define EXIT_LONGDOC "usage: exit\n" \
                 "Exit scanmem immediately, zero will be returned."
 
-bool handler__exit(globals_t * vars, char **argv, unsigned argc);
+bool handler__exit(globals_t *vars, char **argv, unsigned argc);
 
 #define HELP_SHRTDOC "access online documentation, use `help command` for specific help"
 #define HELP_LONGDOC "usage: help [command]\n" \
@@ -160,12 +188,13 @@ bool handler__exit(globals_t * vars, char **argv, unsigned argc);
                 "including options and usage summary. If `command` is not specified, print a\n" \
                 "one line description of each command supported."
 
-bool handler__help(globals_t * vars, char **argv, unsigned argc);
+bool handler__help(globals_t *vars, char **argv, unsigned argc);
 
 #define DEFAULT_SHRTDOC NULL
 #define DEFAULT_LONGDOC "When searching for a number, use any notation in standard C language (leading 0x for\n" \
-                "hexadecimal, leading 0 for octal, everything else is assumed to be decimal)\n" \
-                "float numbers are also acceptable, but will be rounded if scanning integers\n" \
+                "hexadecimal, leading 0 for octal, everything else is assumed to be decimal).\n" \
+                "Float numbers are also acceptable, but will be rounded if scanning integers.\n" \
+                "Use \'..\' for a range, e.g. \'1..3\' searches between 1 and 3 inclusive.\n" \
                 "\n" \
                 "When searching for an array of byte, use 2-byte hexadecimal notation, \n" \
                 "separated by spaces, wildcard '?\?' is also supported. E.g. FF ?\? EE ?\? 02 01\n" \
@@ -177,7 +206,7 @@ bool handler__help(globals_t * vars, char **argv, unsigned argc);
                 "eliminate matches, eventually identifying where the variable is located.\n" \
                 "Once the variable is found, use `set` to change its value.\n"
 
-bool handler__default(globals_t * vars, char **argv, unsigned argc);
+bool handler__default(globals_t *vars, char **argv, unsigned argc);
 
 #define STRING_SHRTDOC "match a given string"
 #define STRING_LONGDOC "usage \" <text>\n" \
@@ -186,7 +215,7 @@ bool handler__default(globals_t * vars, char **argv, unsigned argc);
                 "Example:\n" \
                 "\t\" Scan for string, spaces and ' \" are all acceptable.\n"
 
-bool handler__string(globals_t * vars, char **argv, unsigned argc);
+bool handler__string(globals_t *vars, char **argv, unsigned argc);
 
 #define UPDATE_SHRTDOC "update match values without culling list"
 #define UPDATE_LONGDOC "usage: update\n" \
@@ -195,9 +224,9 @@ bool handler__string(globals_t * vars, char **argv, unsigned argc);
                 "scanmem compares to when using `>`, `<`, or `=`. This command is equivalent\n" \
                 "to a search command that all current results match.\n"
 
-bool handler__update(globals_t * vars, char **argv, unsigned argc);
+bool handler__update(globals_t *vars, char **argv, unsigned argc);
 
-bool handler__eof(globals_t * vars, char **argv, unsigned argc);
+bool handler__eof(globals_t *vars, char **argv, unsigned argc);
 
 #define SHELL_SHRTDOC "execute a shell command without leaving scanmem"
 #define SHELL_LONGDOC "usage: shell [shell-command]\n" \
@@ -208,7 +237,7 @@ bool handler__eof(globals_t * vars, char **argv, unsigned argc);
                 "\tshell man scanmem\n" \
                 "\tshell cat > notes.txt\n"
 
-bool handler__shell(globals_t * vars, char **argv, unsigned argc);
+bool handler__shell(globals_t *vars, char **argv, unsigned argc);
 
 #define WATCH_SHRTDOC "monitor the value of a memory location as it changes"
 #define WATCH_LONGDOC "usage: watch [match-id]\n" \
@@ -218,14 +247,15 @@ bool handler__shell(globals_t * vars, char **argv, unsigned argc);
                 "Examples:\n" \
                 "\twatch 12 - watch match 12 for any changes.\n"
 
-bool handler__watch(globals_t * vars, char **argv, unsigned argc);
+bool handler__watch(globals_t *vars, char **argv, unsigned argc);
 
 /*XXX: improve this */
 #define SHOW_SHRTDOC "display information about scanmem."
 #define SHOW_LONGDOC "usage: show <info>\n" \
-                "Display information relating to <info>.\n"
+                "Display information relating to <info>.\n" \
+                "Possible <info> values: `copying`, `warranty` or `version`\n"
 
-bool handler__show(globals_t * vars, char **argv, unsigned argc);
+bool handler__show(globals_t *vars, char **argv, unsigned argc);
 
 #define DUMP_SHRTDOC "dump a memory region to screen or a file" 
 #define DUMP_LONGDOC "usage: dump <address> <length> [<filename>]\n" \
@@ -233,7 +263,7 @@ bool handler__show(globals_t * vars, char **argv, unsigned argc);
                 "If <filename> is given, save the region of memory to the file \n" \
                 "Otherwise display it in a human-readable format.\n"
     
-bool handler__dump(globals_t * vars, char **argv, unsigned argc);
+bool handler__dump(globals_t *vars, char **argv, unsigned argc);
 
 #define WRITE_SHRTDOC "change the value of a specific memory location"
 #define WRITE_LONGDOC "usage: write <value_type> <address> <value>\n" \
@@ -249,7 +279,7 @@ bool handler__dump(globals_t * vars, char **argv, unsigned argc);
                 "\twrite float32 60103e 0\n" \
                 "\twrite bytearray 60103e ff 01 32\n"
 
-bool handler__write(globals_t * vars, char **argv, unsigned argc);
+bool handler__write(globals_t *vars, char **argv, unsigned argc);
 
 #define OPTION_SHRTDOC "set runtime options of scanmem, see `help option`"
 #define OPTION_LONGDOC "usage: option <option_name> <option_value>\n" \
@@ -292,8 +322,17 @@ bool handler__write(globals_t * vars, char **argv, unsigned argc);
                  "\t0:\tdisabled\n" \
                  "\t1:\tenabled\n" \
                  "\n" \
+                 "endianness\tendianness of data (used by: set, write and comparisons)\n" \
+                 "\t\t\tDefault:0\n" \
+                 "\n" \
+                 "\tpossible values:\n" \
+                 "\t0:\thost endian\n" \
+                 "\t1:\tlittle endian\n" \
+                 "\t2:\tbig endian\n" \
+                 "\n" \
                  "Example:\n" \
                  "\toption scan_data_type int32\n"
 
-bool handler__option(globals_t * vars, char **argv, unsigned argc);
-#endif
+bool handler__option(globals_t *vars, char **argv, unsigned argc);
+
+#endif /* HANDLERS_H */
