@@ -1,23 +1,22 @@
 /*
- $Id: list.c,v 1.12 2007-06-05 01:45:35+01 taviso Exp taviso $
+    A simple linked list implementation.
 
- A simple linked list implementation.
+    Copyright (C) 2006,2007,2009 Tavis Ormandy <taviso@sdf.lonestar.org>
 
- Copyright (C) 2006,2007,2009 Tavis Ormandy <taviso@sdf.lonestar.org>
+    This file is part of libscanmem.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
- (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    This library is free software: you can redistribute it and/or modify
+    it under the terms of the GNU Lesser General Public License as published
+    by the Free Software Foundation; either version 3 of the License, or
+    (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public License
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "config.h"
@@ -49,8 +48,6 @@ void l_destroy(list_t * list)
     free(list);
 }
 
-/*lint -e429 take custody of pointer data */
-
 /* add a new element to the list */
 int l_append(list_t * list, element_t * element, void *data)
 {
