@@ -1,23 +1,25 @@
 /*
- $Id: ptrace.c,v 1.22 2007-06-07 15:16:46+01 taviso Exp taviso $
+    Functions to access the memory of the target process.
  
- Copyright (C) 2006,2007,2009 Tavis Ormandy <taviso@sdf.lonestar.org>
- Copyright (C) 2009           Eli Dupree <elidupree@charter.net>
- Copyright (C) 2009,2010      WANG Lu <coolwanglu@gmail.com>
+    Copyright (C) 2006,2007,2009 Tavis Ormandy <taviso@sdf.lonestar.org>
+    Copyright (C) 2009           Eli Dupree <elidupree@charter.net>
+    Copyright (C) 2009,2010      WANG Lu <coolwanglu@gmail.com>
+    Copyright (C) 2015           Sebastian Parschauer <s.parschauer@gmx.de>
  
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
- (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    This file is part of libscanmem.
+
+    This library is free software: you can redistribute it and/or modify
+    it under the terms of the GNU Lesser General Public License as published
+    by the Free Software Foundation; either version 3 of the License, or
+    (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public License
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "config.h"
@@ -59,11 +61,20 @@
 #endif
 
 #include "value.h"
+#include "endianness.h"
 #include "scanroutines.h"
 #include "scanmem.h"
 #include "show_message.h"
 
-#define min(a,b) (((a)<(b))?(a):(b))
+/* Progress handling */
+#define NUM_DOTS (10)
+#define NUM_SAMPLES (100)
+#define MAX_PROGRESS (1.0)  /* 100% */
+#if (!NUM_DOTS || !NUM_SAMPLES || NUM_SAMPLES % NUM_DOTS != 0)
+#error Invalid NUM_DOTS to NUM_SAMPLES proportion!
+#endif
+#define SAMPLES_PER_DOT (NUM_SAMPLES / NUM_DOTS)
+#define PROGRESS_PER_SAMPLE (MAX_PROGRESS / NUM_SAMPLES)
 
 /* ptrace peek buffer, used by peekdata() */
 /* make it larger in order to reduce shift */
@@ -77,7 +88,7 @@ static struct {
 } peekbuf;
 
 
-bool attach(pid_t target)
+bool sm_attach(pid_t target)
 {
     int status;
 
@@ -102,21 +113,22 @@ bool attach(pid_t target)
 
 }
 
-bool detach(pid_t target)
+bool sm_detach(pid_t target)
 {
     // addr is ignore under Linux, but should be 1 under FreeBSD in order to let the child process continue at what it had been interrupted
     return ptrace(PTRACE_DETACH, target, 1, 0) == 0;
 }
 
+
 /*
- * peekdata - caches overlapping ptrace reads to improve performance.
+ * sm_peekdata - caches overlapping ptrace reads to improve performance.
  * 
  * This routine could just call ptrace(PEEKDATA), but using a cache reduces
  * the number of peeks required by 70% when reading large chunks of
  * consecutive addresses.
  */
 
-bool peekdata(pid_t pid, void *addr, value_t * result)
+bool sm_peekdata(pid_t pid, void *addr, value_t * result)
 {
     char *reqaddr = addr;
     int i, j;
@@ -126,16 +138,15 @@ bool peekdata(pid_t pid, void *addr, value_t * result)
     assert(peekbuf.size <= MAX_PEEKBUF_SIZE);
     assert(result != NULL);
 
-    memset(result, 0x00, sizeof(value_t));
-
+    /* initialise result */
+    zero_value(result);
     valnowidth(result);
 
     /* check if we have a cache hit */
     if (pid == peekbuf.pid &&
             reqaddr >= peekbuf.base &&
             (unsigned long) (reqaddr + sizeof(int64_t) - peekbuf.base) <= peekbuf.size) {
-
-        result->int64_value =    *((int64_t *)&peekbuf.cache[reqaddr - peekbuf.base]);  /*lint !e826 */
+        memcpy(&result->int64_value, &peekbuf.cache[reqaddr - peekbuf.base], sizeof(result->int64_value));
         return true;
     } else if (pid == peekbuf.pid &&
             reqaddr >= peekbuf.base &&
@@ -194,11 +205,11 @@ bool peekdata(pid_t pid, void *addr, value_t * result)
                     /* Cache it with the appropriate offset */
                     if(peekbuf.size >= j)
                     {
-                        *((long *)&peekbuf.cache[peekbuf.size - j]) = ptraced_long;
+                        memcpy(&peekbuf.cache[peekbuf.size - j], &ptraced_long, sizeof(long));
                     }
                     else
                     {
-                        *((long *)&peekbuf.cache[0]) = ptraced_long;
+                        memcpy(&peekbuf.cache[0], &ptraced_long, sizeof(long));
                         peekbuf.base -= j;
                     }
                     peekbuf.size += sizeof(long) - j;
@@ -214,7 +225,7 @@ bool peekdata(pid_t pid, void *addr, value_t * result)
         }
         
         /* Otherwise, ptrace() worked - cache the data, increase the size */
-        *((long *)&peekbuf.cache[peekbuf.size]) = ptraced_long;
+        memcpy(&peekbuf.cache[peekbuf.size], &ptraced_long, sizeof(long));
         peekbuf.size += sizeof(long);
         last_address_gathered = ptrace_address + sizeof(long);
     }
@@ -225,7 +236,7 @@ bool peekdata(pid_t pid, void *addr, value_t * result)
     if (reqaddr + sizeof(int64_t) <= last_address_gathered)
     {
         /* The values are fine - read away */
-        result->int64_value =    *((int64_t *)&peekbuf.cache[reqaddr - peekbuf.base]);  /*lint !e826 */
+        memcpy(&result->int64_value, &peekbuf.cache[reqaddr - peekbuf.base], sizeof(result->int64_value));
     }
     else
     {
@@ -253,11 +264,17 @@ bool peekdata(pid_t pid, void *addr, value_t * result)
     return true;
 }
 
+static inline void print_a_dot(void)
+{
+    fprintf(stderr, ".");
+    fflush(stderr);
+}
+
 /* This is the function that handles when you enter a value (or >, <, =) for the second or later time (i.e. when there's already a list of matches); it reduces the list to those that still match. It returns false on failure to attach, detach, or reallocate memory, otherwise true.
 "value" is what to compare to. It is meaningless when the match type is not MATCHEXACT. */
-bool checkmatches(globals_t * vars, 
-                  scan_match_type_t match_type,
-                  const uservalue_t *uservalue) 
+bool sm_checkmatches(globals_t *vars,
+                     scan_match_type_t match_type,
+                     const uservalue_t *uservalue)
 {
     matches_and_old_values_swath *reading_swath_index = (matches_and_old_values_swath *)vars->matches->swaths;
     matches_and_old_values_swath reading_swath = *reading_swath_index;
@@ -265,11 +282,20 @@ bool checkmatches(globals_t * vars,
     long bytes_scanned = 0;
     long total_scan_bytes = 0;
     matches_and_old_values_swath *tmp_swath_index = reading_swath_index;
+    int samples_remaining = NUM_SAMPLES;
+    int samples_to_dot = SAMPLES_PER_DOT;
+    size_t bytes_at_next_sample;
+    size_t bytes_per_sample;
+
     while(tmp_swath_index->number_of_bytes)
     {
         total_scan_bytes += tmp_swath_index->number_of_bytes;
         tmp_swath_index = (matches_and_old_values_swath *)(&tmp_swath_index->data[tmp_swath_index->number_of_bytes]);
     }
+    bytes_per_sample = total_scan_bytes / NUM_SAMPLES;
+    bytes_at_next_sample = bytes_per_sample;
+    /* for user, just print the first dot */
+    print_a_dot();
 
     int reading_iterator = 0;
     matches_and_old_values_swath *writing_swath_index = (matches_and_old_values_swath *)vars->matches->swaths;
@@ -278,21 +304,22 @@ bool checkmatches(globals_t * vars,
     
     /* used to fill in non-match regions */
     match_flags zero_flag;
-    memset(&zero_flag, 0, sizeof(zero_flag));
+    zero_match_flags(&zero_flag);
 
     int required_extra_bytes_to_record = 0;
     vars->num_matches = 0;
+    vars->scan_progress = 0.0;
     
-    if (choose_scanroutine(vars->options.scan_data_type, match_type) == false)
+    if (sm_choose_scanroutine(vars->options.scan_data_type, match_type) == false)
     {
         show_error("unsupported scan for current data type.\n"); 
         return false;
     }
 
-    assert(g_scan_routine);
+    assert(sm_scan_routine);
     
     /* stop and attach to the target */
-    if (attach(vars->target) == false)
+    if (sm_attach(vars->target) == false)
         return false;
     while (reading_swath.first_byte_in_child) {
         int match_length = 0;
@@ -302,7 +329,7 @@ bool checkmatches(globals_t * vars,
         void *address = reading_swath.first_byte_in_child + reading_iterator;
         
         /* Read value from this address */
-        if (EXPECT(peekdata(vars->target, address, &data_value) == false, false)) {
+        if (EXPECT(sm_peekdata(vars->target, address, &data_value) == false, false)) {
             /* Uhh, what? We couldn't look at the data there? I guess this doesn't count as a match then */
         }
         else
@@ -314,9 +341,11 @@ bool checkmatches(globals_t * vars,
             truncval_to_flags(&old_val, flags);
             truncval_to_flags(&data_value, flags);
 
-            memset(&checkflags, 0, sizeof(checkflags));
+            fix_endianness(vars, &data_value);
 
-            match_length = (*g_scan_routine)(&data_value, &old_val, uservalue, &checkflags, address);
+            zero_match_flags(&checkflags);
+
+            match_length = (*sm_scan_routine)(&data_value, &old_val, uservalue, &checkflags, address);
         }
         
         if (match_length > 0)
@@ -339,18 +368,27 @@ bool checkmatches(globals_t * vars,
             --required_extra_bytes_to_record;
         }
 
-        if (EXPECT((total_scan_bytes >= 110) && (bytes_scanned % ((total_scan_bytes) / 10) == 10), false)) {
-            /* for user, just print a dot */
-            show_scan_progress(bytes_scanned, total_scan_bytes);
+        if (EXPECT(bytes_scanned >= bytes_at_next_sample, false)) {
+            bytes_at_next_sample += bytes_per_sample;
+            /* handle rounding */
+            if (EXPECT(--samples_remaining > 0, true)) {
+                /* for front-end, update percentage */
+                vars->scan_progress += PROGRESS_PER_SAMPLE;
+                if (EXPECT(--samples_to_dot == 0, false)) {
+                    samples_to_dot = SAMPLES_PER_DOT;
+                    /* for user, just print a dot */
+                    print_a_dot();
+                }
+            }
         }
-        ++ bytes_scanned;
+        ++bytes_scanned;
         
         /* Go on to the next one... */
         ++reading_iterator;
         if (reading_iterator >= reading_swath.number_of_bytes)
         {
-            assert(((matches_and_old_values_swath *)(&reading_swath_index->data[reading_swath.number_of_bytes]))->number_of_bytes >= 0);
-            reading_swath_index = (matches_and_old_values_swath *)(&reading_swath_index->data[reading_swath.number_of_bytes]);
+            reading_swath_index = (matches_and_old_values_swath *)
+                (&reading_swath_index->data[reading_swath.number_of_bytes]);
             reading_swath = *reading_swath_index;
             reading_iterator = 0;
             required_extra_bytes_to_record = 0; /* just in case */
@@ -363,17 +401,13 @@ bool checkmatches(globals_t * vars,
         return false;
     }
 
-    /* hack for front-end, it needs this information */
-    /* TODO: we'll need progress for checkmatches too */
-    if (vars->options.backend == 1)
-    {
-        show_scan_progress(total_scan_bytes, total_scan_bytes);
-    }
+    /* tell front-end we've done */
+    vars->scan_progress = MAX_PROGRESS;
 
     show_info("we currently have %ld matches.\n", vars->num_matches);
 
     /* okay, detach */
-    return detach(vars->target);
+    return sm_detach(vars->target);
 }
 
 /* read region using /proc/pid/mem */
@@ -402,8 +436,8 @@ ssize_t readregion(pid_t target, void *buf, size_t count, unsigned long offset)
 }
     
 
-/* searchregions() performs an initial search of the process for values matching value */
-bool searchregions(globals_t * vars, scan_match_type_t match_type, const uservalue_t *uservalue)
+/* sm_searchregions() performs an initial search of the process for values matching value */
+bool sm_searchregions(globals_t *vars, scan_match_type_t match_type, const uservalue_t *uservalue)
 {
     matches_and_old_values_swath *writing_swath_index;
     int required_extra_bytes_to_record = 0;
@@ -412,7 +446,6 @@ bool searchregions(globals_t * vars, scan_match_type_t match_type, const userval
     element_t *n = vars->regions->head;
     region_t *r;
     unsigned long total_scan_bytes = 0;
-    unsigned long bytes_scanned = 0;
     void *address = NULL;
 
 #if HAVE_PROCMEM
@@ -422,18 +455,18 @@ bool searchregions(globals_t * vars, scan_match_type_t match_type, const userval
 
     /* used to fill in non-match regions */
     match_flags zero_flag;
-    memset(&zero_flag, 0, sizeof(zero_flag));
+    zero_match_flags(&zero_flag);
     
-    if (choose_scanroutine(vars->options.scan_data_type, match_type) == false)
+    if (sm_choose_scanroutine(vars->options.scan_data_type, match_type) == false)
     {
         show_error("unsupported scan for current data type.\n"); 
         return false;
     }
 
-    assert(g_scan_routine);
+    assert(sm_scan_routine);
 
     /* stop and attach to the target */
-    if (attach(vars->target) == false)
+    if (sm_attach(vars->target) == false)
         return false;
 
    
@@ -441,7 +474,7 @@ bool searchregions(globals_t * vars, scan_match_type_t match_type, const userval
     if (vars->regions->size == 0) {
         show_warn("no regions defined, perhaps you deleted them all?\n");
         show_info("use the \"reset\" command to refresh regions.\n");
-        return detach(vars->target);
+        return sm_detach(vars->target);
     }
     
     total_size = sizeof(matches_and_old_values_array);
@@ -470,14 +503,22 @@ bool searchregions(globals_t * vars, scan_match_type_t match_type, const userval
     for(n = vars->regions->head; n; n = n->next)
         total_scan_bytes += ((region_t *)n->data)->size;
 
+    vars->scan_progress = 0.0;
     n = vars->regions->head;
 
     /* check every memory region */
     while (n) {
         unsigned offset, nread = 0;
+        int dots_remaining = NUM_DOTS;
+        size_t bytes_at_next_dot;
+        size_t bytes_per_dot;
+        double progress_per_dot;
 
         /* load the next region */
         r = n->data;
+        bytes_per_dot = r->size / NUM_DOTS;
+        bytes_at_next_dot = bytes_per_dot;
+        progress_per_dot = (double)bytes_per_dot / total_scan_bytes;
 
 #if HAVE_PROCMEM        
         /* over allocate by enough bytes set to zero that the last bytes can be read as 64-bit ints */
@@ -503,7 +544,7 @@ bool searchregions(globals_t * vars, scan_match_type_t match_type, const userval
 #endif
         /* print a progress meter so user knows we havent crashed */
         /* cannot use show_info here because it'll append a '\n' */
-        show_info("%02u/%02u searching %#10lx - %#10lx.", ++regnum,
+        show_user("%02u/%02u searching %#10lx - %#10lx.", ++regnum,
                 vars->regions->size, (unsigned long)r->start, (unsigned long)r->start + r->size);
         fflush(stderr);
 
@@ -512,16 +553,17 @@ bool searchregions(globals_t * vars, scan_match_type_t match_type, const userval
             match_flags checkflags;
             value_t data_value;
            
-            /* initialise check */
-            memset(&data_value, 0x00, sizeof(data_value));
-            
+            /* initialise data_value */
+            zero_value(&data_value);
             valnowidth(&data_value);
 
             address = r->start + offset;
 
-#if HAVE_PROCMEM           
-            data_value.int64_value    = *((int64_t *)(&data[offset]));
-            
+#if HAVE_PROCMEM
+            /* Don't dereference as this causes an alignment issue e.g. on ARM.
+               GCC replaces memcpy() with dereferencing where possible. */
+            memcpy(&data_value.int64_value, &data[offset], sizeof(int64_t));
+
             /* Mark which values this can't be */
             if (EXPECT((nread - offset < sizeof(int64_t)), false))
             {
@@ -545,14 +587,18 @@ bool searchregions(globals_t * vars, scan_match_type_t match_type, const userval
             }
 #endif
 
-            memset(&checkflags, 0, sizeof(checkflags));
+            fix_endianness(vars, &data_value);
+
+            /* initialise checkflags */
+            zero_match_flags(&checkflags);
 
             int match_length;
             /* check if we have a match */
-            if (EXPECT(((match_length = (*g_scan_routine)(&data_value, NULL, uservalue, &checkflags, address)) > 0), false)) {
+            if (EXPECT(((match_length = (*sm_scan_routine)(&data_value, NULL,
+                    uservalue, &checkflags, address)) > 0), false)) {
                 /* only set these flags for numbers */
-                if ((globals.options.scan_data_type != BYTEARRAY)
-                    && (globals.options.scan_data_type != STRING))
+                if ((vars->options.scan_data_type != BYTEARRAY)
+                    && (vars->options.scan_data_type != STRING))
                 {
                     checkflags.ineq_forwards = checkflags.ineq_reverse = 1;
                 }
@@ -571,13 +617,19 @@ bool searchregions(globals_t * vars, scan_match_type_t match_type, const userval
             }
 
             /* print a simple progress meter. */
-            if (EXPECT((r->size >= 110) && (offset % ((r->size) / 10) == 10), false)) {
-                /* for user, just print a dot */
-                show_scan_progress(bytes_scanned+offset, total_scan_bytes);
+            if (EXPECT(offset >= bytes_at_next_dot, false)) {
+                bytes_at_next_dot += bytes_per_dot;
+                /* handle rounding */
+                if (EXPECT(--dots_remaining > 0, true)) {
+                    /* for user, just print a dot */
+                    print_a_dot();
+                    /* for front-end, update percentage */
+                    vars->scan_progress += progress_per_dot;
+                }
             }
         }
 
-        bytes_scanned += r->size;
+        vars->scan_progress += progress_per_dot;
         n = n->next;
         show_user("ok\n");
 #if HAVE_PROCMEM
@@ -586,8 +638,7 @@ bool searchregions(globals_t * vars, scan_match_type_t match_type, const userval
     }
 
     /* tell front-end we've done */
-    if(vars->options.backend == 1)
-        show_scan_progress(total_scan_bytes, total_scan_bytes);
+    vars->scan_progress = MAX_PROGRESS;
     
     if (!(vars->matches = null_terminate(vars->matches, writing_swath_index /* ,MATCHES_AND_VALUES */)))
     {
@@ -598,19 +649,19 @@ bool searchregions(globals_t * vars, scan_match_type_t match_type, const userval
     show_info("we currently have %ld matches.\n", vars->num_matches);
 
     /* okay, detach */
-    return detach(vars->target);
+    return sm_detach(vars->target);
 }
 
-bool setaddr(pid_t target, void *addr, const value_t * to)
+bool sm_setaddr(pid_t target, void *addr, const value_t *to)
 {
     value_t saved;
     int i;
 
-    if (attach(target) == false) {
+    if (sm_attach(target) == false) {
         return false;
     }
 
-    if (peekdata(target, addr, &saved) == false) {
+    if (sm_peekdata(target, addr, &saved) == false) {
         show_error("couldnt access the target address %10p\n", addr);
         return false;
     }
@@ -641,12 +692,12 @@ bool setaddr(pid_t target, void *addr, const value_t * to)
         }
     }
 
-    return detach(target);
+    return sm_detach(target);
 }
 
-bool read_array(pid_t target, void *addr, char *buf, int len)
+bool sm_read_array(pid_t target, void *addr, char *buf, int len)
 {
-    if (attach(target) == false) {
+    if (sm_attach(target) == false) {
         return false;
     }
 
@@ -665,11 +716,11 @@ bool read_array(pid_t target, void *addr, char *buf, int len)
 
     if (nread < len)
     {
-        detach(target);
+        sm_detach(target);
         return false;
     }
 
-    return detach(target);
+    return sm_detach(target);
 #else
     int i;
     /* here we just read long by long, this should be ok for most of time */
@@ -679,21 +730,21 @@ bool read_array(pid_t target, void *addr, char *buf, int len)
         errno = 0;
         *((long *)(buf+i)) = ptrace(PTRACE_PEEKDATA, target, addr+i, NULL);
         if (EXPECT((*((long *)(buf+i)) == -1L) && (errno != 0), false)) {
-            detach(target);
+            sm_detach(target);
             return false;
         }
     }
-    return detach(target);
+    return sm_detach(target);
 #endif
 }
 
 /* TODO: may use /proc/<pid>/mem here */
-bool write_array(pid_t target, void *addr, const void *data, int len)
+bool sm_write_array(pid_t target, void *addr, const void *data, int len)
 {
     int i,j;
     long peek_value;
 
-    if (attach(target) == false) {
+    if (sm_attach(target) == false) {
         return false;
     }
 
@@ -724,7 +775,7 @@ bool write_array(pid_t target, void *addr, const void *data, int len)
                         continue;
                     else
                     {
-                        show_error("write_array failed.\n"); 
+                        show_error("%s failed.\n", __func__);
                         return false;
                     }
                 }
@@ -735,7 +786,7 @@ bool write_array(pid_t target, void *addr, const void *data, int len)
 
                     if (ptrace(PTRACE_POKEDATA, target, addr - j, peek_value) == -1L)
                     {
-                        show_error("write_array failed.\n");
+                        show_error("%s failed.\n", __func__);
                         return false;
                     }
 
@@ -745,5 +796,5 @@ bool write_array(pid_t target, void *addr, const void *data, int len)
         }
     }
 
-    return detach(target);
+    return sm_detach(target);
 }
