@@ -1,19 +1,22 @@
 /*
-    scanroutines.c  Routines of scanning for different data types
+    Routines of scanning for different data types.
+
     Copyright (C) 2009,2010 WANG Lu  <coolwanglu(a)gmail.com>
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    This file is part of libscanmem.
+
+    This library is free software: you can redistribute it and/or modify
+    it under the terms of the GNU Lesser General Public License as published
+    by the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
+    This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    You should have received a copy of the GNU Lesser General Public License
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "config.h"
@@ -27,7 +30,7 @@
 
 /* for convenience */
 #define SCAN_ROUTINE_ARGUMENTS (const value_t *new_value, const value_t *old_value, const uservalue_t *user_value, match_flags *saveflags, void *address) 
-int (*g_scan_routine) SCAN_ROUTINE_ARGUMENTS;
+int (*sm_scan_routine) SCAN_ROUTINE_ARGUMENTS;
 
 #define VALUE_COMP(a,b,field,op)    (((a)->flags.field && (b)->flags.field) && (get_##field(a) op get_##field(b)))
 #define VALUE_COPY(a,b,field)       ((set_##field(a, get_##field(b))), ((a)->flags.field = 1))
@@ -131,13 +134,12 @@ DEFINE_FLOAT_ROUTINE_FOR_ALL_FLOAT_TYPE(LESSTHAN, <, user_value)
 /*-----------------------------------*/
 /* special EQUALTO for float numbers */
 /*-----------------------------------*/
-/* currently we round both of them to integers and compare them */
-/* TODO: let user specify a float number */
 #define DEFINE_FLOAT_EQUALTO_ROUTINE(FLOATTYPENAME, WIDTH) \
     int scan_routine_##FLOATTYPENAME##_EQUALTO SCAN_ROUTINE_ARGUMENTS \
     { \
         int ret = 0; \
-        if((new_value->flags.f##WIDTH##b) && (user_value->flags.f##WIDTH##b) && ((int##WIDTH##_t)get_f##WIDTH##b(new_value) == (int##WIDTH##_t)get_f##WIDTH##b(user_value))) \
+        if ((new_value->flags.f##WIDTH##b) && (user_value->flags.f##WIDTH##b) \
+            && (get_f##WIDTH##b(new_value) == get_f##WIDTH##b(user_value))) \
         { \
             SET_FLAG(saveflags, f##WIDTH##b); \
             ret = (WIDTH)/8; \
@@ -224,9 +226,50 @@ DEFINE_INTEGER_INCREASEDBY_DECREASEDBY_ROUTINE(INTEGER64, 64)
 DEFINE_FLOAT_INCREASEDBY_DECREASEDBY_ROUTINE(FLOAT32, 32)
 DEFINE_FLOAT_INCREASEDBY_DECREASEDBY_ROUTINE(FLOAT64, 64)
 
-/*----------------*/
+/*-----------*/
+/* for RANGE */
+/*-----------*/
+
+#define DEFINE_INTEGER_RANGE_ROUTINE(DATATYPENAME, DATAWIDTH) \
+    int scan_routine_##DATATYPENAME##_RANGE SCAN_ROUTINE_ARGUMENTS \
+    { \
+        int ret = 0; \
+        if ((new_value->flags.s##DATAWIDTH##b) \
+                && (user_value[0].flags.s##DATAWIDTH##b) && (user_value[1].flags.s##DATAWIDTH##b) \
+                && (get_s##DATAWIDTH##b(new_value) >= get_s##DATAWIDTH##b(&user_value[0])) \
+                && (get_s##DATAWIDTH##b(new_value) <= get_s##DATAWIDTH##b(&user_value[1]))) \
+            { ret = (DATAWIDTH)/8; SET_FLAG(saveflags, s##DATAWIDTH##b); } \
+        if ((new_value->flags.u##DATAWIDTH##b) \
+                && (user_value[0].flags.u##DATAWIDTH##b) && (user_value[1].flags.u##DATAWIDTH##b) \
+                && (get_u##DATAWIDTH##b(new_value) >= get_u##DATAWIDTH##b(&user_value[0])) \
+                && (get_u##DATAWIDTH##b(new_value) <= get_u##DATAWIDTH##b(&user_value[1]))) \
+            { ret = (DATAWIDTH)/8; SET_FLAG(saveflags, u##DATAWIDTH##b); } \
+        return ret; \
+    }
+
+DEFINE_INTEGER_RANGE_ROUTINE(INTEGER8, 8)
+DEFINE_INTEGER_RANGE_ROUTINE(INTEGER16, 16)
+DEFINE_INTEGER_RANGE_ROUTINE(INTEGER32, 32)
+DEFINE_INTEGER_RANGE_ROUTINE(INTEGER64, 64)
+
+#define DEFINE_FLOAT_RANGE_ROUTINE(DATATYPENAME, DATAWIDTH) \
+    int scan_routine_##DATATYPENAME##_RANGE SCAN_ROUTINE_ARGUMENTS \
+    { \
+        int ret = 0; \
+        if ((new_value->flags.f##DATAWIDTH##b) \
+                && (user_value[0].flags.f##DATAWIDTH##b) && (user_value[1].flags.f##DATAWIDTH##b) \
+                && (get_f##DATAWIDTH##b(new_value) >= get_f##DATAWIDTH##b(&user_value[0])) \
+                && (get_f##DATAWIDTH##b(new_value) <= get_f##DATAWIDTH##b(&user_value[1]))) \
+            { ret = (DATAWIDTH)/8; SET_FLAG(saveflags, f##DATAWIDTH##b); } \
+        return ret; \
+    }
+
+DEFINE_FLOAT_RANGE_ROUTINE(FLOAT32, 32)
+DEFINE_FLOAT_RANGE_ROUTINE(FLOAT64, 64)
+
+/*---------------*/
 /* for BYTEARRAY */
-/*----------------*/
+/*---------------*/
 int scan_routine_BYTEARRAY_ANY SCAN_ROUTINE_ARGUMENTS
 {
    return saveflags->bytearray_length = ((old_value)->flags.bytearray_length); 
@@ -258,7 +301,7 @@ int scan_routine_BYTEARRAY_EQUALTO SCAN_ROUTINE_ARGUMENTS
         } 
          
         /* read next block */
-        if (!peekdata(globals.target, address+i+sizeof(int64_t), &val_buf))
+        if (!sm_peekdata(sm_globals.target, address+i+sizeof(int64_t), &val_buf))
         {
             /* cannot read */
             return 0;
@@ -309,7 +352,7 @@ int scan_routine_STRING_EQUALTO SCAN_ROUTINE_ARGUMENTS
         } 
          
         /* read next block */
-        if (!peekdata(globals.target, address+i+sizeof(int64_t), &val_buf))
+        if (!sm_peekdata(sm_globals.target, address + i + sizeof(int64_t), &val_buf))
         {
             /* cannot read */
             return 0;
@@ -372,7 +415,7 @@ DEFINE_ANYTYPE_ROUTINE(INCREASED_WITH_REVERSE)
 DEFINE_ANYTYPE_ROUTINE(DECREASED_WITH_REVERSE)
 DEFINE_ANYTYPE_ROUTINE(INCREASEDBY)
 DEFINE_ANYTYPE_ROUTINE(DECREASEDBY)
-
+DEFINE_ANYTYPE_ROUTINE(RANGE)
 
 
 /***************************************************************/
@@ -400,14 +443,14 @@ DEFINE_ANYTYPE_ROUTINE(DECREASEDBY)
 
 
 
-bool choose_scanroutine(scan_data_type_t dt, scan_match_type_t mt)
+bool sm_choose_scanroutine(scan_data_type_t dt, scan_match_type_t mt)
 {
-    return (g_scan_routine = get_scanroutine(dt, mt)) != NULL;
+    return (sm_scan_routine = sm_get_scanroutine(dt, mt)) != NULL;
 }
 
-scan_routine_t get_scanroutine(scan_data_type_t dt, scan_match_type_t mt)
+scan_routine_t sm_get_scanroutine(scan_data_type_t dt, scan_match_type_t mt)
 {
-    if (globals.options.detect_reverse_change)
+    if (sm_globals.options.detect_reverse_change)
     {
         CHOOSE_ROUTINE_FOR_ALL_NUMBER_TYPES(MATCHINCREASED, INCREASED_WITH_REVERSE)
         CHOOSE_ROUTINE_FOR_ALL_NUMBER_TYPES(MATCHDECREASED, DECREASED_WITH_REVERSE)
@@ -424,11 +467,10 @@ scan_routine_t get_scanroutine(scan_data_type_t dt, scan_match_type_t mt)
     CHOOSE_ROUTINE_FOR_ALL_NUMBER_TYPES(MATCHDECREASEDBY, DECREASEDBY)
     CHOOSE_ROUTINE_FOR_ALL_NUMBER_TYPES(MATCHGREATERTHAN, GREATERTHAN)
     CHOOSE_ROUTINE_FOR_ALL_NUMBER_TYPES(MATCHLESSTHAN, LESSTHAN)
+    CHOOSE_ROUTINE_FOR_ALL_NUMBER_TYPES(MATCHRANGE, RANGE)
 
     CHOOSE_ROUTINE(BYTEARRAY, BYTEARRAY, MATCHEQUALTO, EQUALTO) 
     CHOOSE_ROUTINE(STRING, STRING, MATCHEQUALTO, EQUALTO) 
 
     return NULL;
 }
-
-
